@@ -924,7 +924,7 @@ static irqreturn_t me4000_ai_isr(int irq, void *dev_id)
 	struct comedi_subdevice *s = dev->read_subdev;
 	int i;
 	int c = 0;
-	unsigned short lval;
+	unsigned int lval;
 
 	if (!dev->attached)
 		return IRQ_NONE;
